@@ -13,18 +13,15 @@ from pycalphad.core.utils import broadcast_to, unpack_condition, unpack_phases
 from pycalphad.core.cache import cacheit
 from pycalphad.core.phase_rec import PhaseRecord, PhaseRecord_from_cython, PhaseRecord_from_compiledmodel
 from pycalphad.core.compiled_model import CompiledModel
-from pycalphad.log import logger
 import pycalphad.variables as v
 from sympy import Symbol
 import numpy as np
 import itertools
 import collections
 import warnings
-from xarray import Dataset, Variable
-from xarray.core.dataset import as_dataset
-from xarray.core.combine import _calc_concat_over, _calc_concat_dim_coord, concat_vars
+from xarray import Dataset, concat
 from collections import OrderedDict
-import pandas as pd
+
 
 class FallbackModel(object):
     "Compatibility layer while transitioning to CompiledModel."
@@ -35,80 +32,6 @@ class FallbackModel(object):
             return Model(*args, **kwargs)
         return ret
 
-def _fast_concat(datasets, dim=None):
-    """
-    Fork of xarray.concat for Datasets.
-    This function assumes everything is already aligned, and skips error checking.
-
-    Parameters
-    ----------
-    datasets
-
-    Returns
-    -------
-
-    """
-    data_vars = 'all'
-    coords = 'different'
-    positions = None
-
-    dim, coord = _calc_concat_dim_coord(dim)
-    datasets = [as_dataset(ds) for ds in datasets]
-
-    concat_over = _calc_concat_over(datasets, dim, data_vars, coords)
-
-    def insert_result_variable(k, v):
-        assert isinstance(v, Variable)
-        if k in datasets[0].coords:
-            result_coord_names.add(k)
-        result_vars[k] = v
-
-    # create the new dataset and add constant variables
-    result_vars = OrderedDict()
-    result_coord_names = set(datasets[0].coords)
-    result_attrs = datasets[0].attrs
-
-    for k, v in datasets[0].variables.items():
-        if k not in concat_over:
-            insert_result_variable(k, v)
-
-    # we've already verified everything is consistent; now, calculate
-    # shared dimension sizes so we can expand the necessary variables
-    dim_lengths = [ds.dims.get(dim, 1) for ds in datasets]
-    non_concat_dims = {}
-    for ds in datasets:
-        non_concat_dims.update(ds.dims)
-    non_concat_dims.pop(dim, None)
-
-    def ensure_common_dims(vars):
-        # ensure each variable with the given name shares the same
-        # dimensions and the same shape for all of them except along the
-        # concat dimension
-        common_dims = tuple(pd.unique([d for v in vars for d in v.dims]))
-        if dim not in common_dims:
-            common_dims = (dim,) + common_dims
-        for var, dim_len in zip(vars, dim_lengths):
-            if var.dims != common_dims:
-                common_shape = tuple(non_concat_dims.get(d, dim_len)
-                                     for d in common_dims)
-                var = var.expand_dims(common_dims, common_shape)
-            yield var
-
-    # stack up each variable to fill-out the dataset
-    for k in concat_over:
-        vars = ensure_common_dims([ds.variables[k] for ds in datasets])
-        combined = concat_vars(vars, dim, positions)
-        insert_result_variable(k, combined)
-
-    result = Dataset(result_vars, attrs=result_attrs)
-    result = result.set_coords(result_coord_names)
-
-    if coord is not None:
-        # add concat dimension last to ensure that its in the final Dataset
-        result[coord.name] = coord
-
-    return result
-
 
 def _generate_fake_points(components, statevar_dict, energy_limit, output, maximum_internal_dof, broadcast):
     """
@@ -423,13 +346,14 @@ def calculate(dbf, comps, phases, mode=None, output='GM', fake_points=False, bro
 
     # Convert keyword strings to proper state variable objects
     # If we don't do this, sympy will get confused during substitution
-    statevar_dict = collections.OrderedDict((v.StateVariable(key), unpack_condition(value)) \
-                                            for (key, value) in sorted(kwargs.items()))
+    statevar_dict = dict((v.StateVariable(key), unpack_condition(value)) for (key, value) in kwargs.items())
     # XXX: CompiledModel assumes P, T are the only state variables
     if statevar_dict.get(v.P, None) is None:
         statevar_dict[v.P] = 101325
     if statevar_dict.get(v.T, None) is None:
         statevar_dict[v.T] = 300
+    # Sort after default state variable check to fix gh-116
+    statevar_dict = collections.OrderedDict(sorted(statevar_dict.items(), key=lambda x: str(x[0])))
     str_statevar_dict = collections.OrderedDict((str(key), unpack_condition(value)) \
                                                 for (key, value) in statevar_dict.items())
     all_phase_data = []
@@ -452,9 +376,8 @@ def calculate(dbf, comps, phases, mode=None, output='GM', fake_points=False, bro
                 # we can't build the specified phase because the
                 # specified components aren't found in every sublattice
                 # we'll just skip it
-                logger.warning("""Suspending specified phase %s due to
-                some sublattices containing only unspecified components""",
-                               phase_name)
+                warnings.warn("""Suspending specified phase {} due to
+                some sublattices containing only unspecified components""".format(phase_name))
                 continue
         if points_dict[phase_name] is None:
             maximum_internal_dof = max(maximum_internal_dof, sum(len(x) for x in mod.constituents))
@@ -516,7 +439,7 @@ def calculate(dbf, comps, phases, mode=None, output='GM', fake_points=False, bro
 
     # speedup for single-phase case (found by profiling)
     if len(all_phase_data) > 1:
-        final_ds = _fast_concat(all_phase_data, dim='points')
+        final_ds = concat(all_phase_data, dim='points')
         final_ds['points'].values = np.arange(len(final_ds['points']))
         final_ds.coords['points'].values = np.arange(len(final_ds['points']))
     else:
