@@ -105,6 +105,8 @@ def import_extension(path, modname):
     # We filter out any files that can be created by compilers which are not our actual compiled file.
     # We cannot more directly search for our files because of differing platforms.
     blacklist = ['dSYM', 'c', 'pyx']
+    if os.name == 'nt':
+        blacklist.extend(['def', 'o'])
     npath = [x for x in npath if x.split('.')[-1] not in blacklist]
     if len(npath) == 1:
         npath = npath[0]
