@@ -6,7 +6,7 @@ from __future__ import print_function
 import warnings
 import pycalphad.variables as v
 from pycalphad.core.utils import unpack_kwarg
-from pycalphad.core.utils import unpack_condition, unpack_phases
+from pycalphad.core.utils import unpack_condition, unpack_phases, filter_phases
 from pycalphad import calculate, Model
 from pycalphad.core.lower_convex_hull import lower_convex_hull
 from pycalphad.core.sympydiff_utils import build_functions as compiled_build_functions
@@ -18,7 +18,12 @@ from pycalphad.core.eqsolver import _solve_eq_at_conditions
 from sympy import Add, Symbol
 import dask
 from dask import delayed
-import dask.multiprocessing, dask.async
+import dask.multiprocessing
+try:
+    import dask.local
+except ImportError:
+    import dask.async
+    dask.local = dask.async
 from xarray import Dataset
 import numpy as np
 from collections import namedtuple, OrderedDict
@@ -26,12 +31,12 @@ from datetime import datetime
 
 
 class EquilibriumError(Exception):
-    "Exception related to calculation of equilibrium"
+    "Exception related to calculation of equilibrium."
     pass
 
 
 class ConditionError(EquilibriumError):
-    "Exception related to equilibrium conditions"
+    "Exception related to equilibrium conditions."
     pass
 
 
@@ -47,7 +52,7 @@ def _adjust_conditions(conds):
 
 
 def _merge_property_slices(properties, chunk_grid, slices, conds_keys, results):
-    "Merge back together slices of 'properties'"
+    "Merge back together slices of 'properties'."
     for prop_slice, prop_arr in zip(chunk_grid, results):
         if not isinstance(prop_arr, Dataset):
             print('Error: {}'.format(prop_arr))
@@ -146,7 +151,7 @@ def _eqcalculate(dbf, comps, phases, conditions, output, data=None, per_phase=Fa
 
 def equilibrium(dbf, comps, phases, conditions, output=None, model=None,
                 verbose=False, broadcast=True, calc_opts=None,
-                scheduler=dask.async.get_sync,
+                scheduler=dask.local.get_sync,
                 parameters=None, **kwargs):
     """
     Calculate the equilibrium state of a system containing the specified
@@ -193,7 +198,14 @@ def equilibrium(dbf, comps, phases, conditions, output=None, model=None,
     if not broadcast:
         raise NotImplementedError('Broadcasting cannot yet be disabled')
     from pycalphad import __version__ as pycalphad_version
-    active_phases = unpack_phases(phases) or sorted(dbf.phases.keys())
+    phases = unpack_phases(phases) or sorted(dbf.phases.keys())
+    # remove phases that cannot be active
+    list_of_possible_phases = filter_phases(dbf, comps)
+    active_phases = sorted(set(list_of_possible_phases).intersection(set(phases)))
+    if len(list_of_possible_phases) == 0:
+        raise ConditionError('There are no phases in the Database that can be active with components {0}'.format(comps))
+    if len(active_phases) == 0:
+        raise ConditionError('None of the passed phases ({0}) are active. List of possible phases: {1}.'.format(phases, list_of_possible_phases))
     comps = sorted(comps)
     if len(set(comps) - set(dbf.elements)) > 0:
         raise EquilibriumError('Components not found in database: {}'.format(','.join(set(comps) - set(dbf.elements))))
@@ -277,7 +289,6 @@ def equilibrium(dbf, comps, phases, conditions, output=None, model=None,
     grid_opts.update({key: value for key, value in str_conds.items() if key in indep_vars})
     if 'pdens' not in grid_opts:
         grid_opts['pdens'] = 500
-
     coord_dict = str_conds.copy()
     coord_dict['vertex'] = np.arange(len(components))
     grid_shape = np.meshgrid(*coord_dict.values(),
@@ -356,7 +367,7 @@ def equilibrium(dbf, comps, phases, conditions, output=None, model=None,
         properties = delayed(properties.merge, pure=False)(eqcal, inplace=True, compat='equals')
     if scheduler is not None:
         properties = dask.compute(properties, get=scheduler)[0]
-    properties.attrs['created'] = datetime.utcnow()
+    properties.attrs['created'] = datetime.utcnow().isoformat()
     if len(kwargs) > 0:
         warnings.warn('The following equilibrium keyword arguments were passed, but unused:\n{}'.format(kwargs))
     return properties
