@@ -1,4 +1,5 @@
-"""The tdb module provides support for reading and writing databases in
+"""
+The tdb module provides support for reading and writing databases in
 Thermo-Calc TDB format.
 """
 
@@ -14,7 +15,7 @@ from sympy.printing.str import StrPrinter
 from sympy.core.mul import _keep_coeff
 from sympy.printing.precedence import precedence
 from pycalphad import Database
-from pycalphad.io.database import DatabaseExportError
+from pycalphad.io.database import DatabaseExportError, Species
 import pycalphad.variables as v
 from pycalphad.io.tdb_keywords import expand_keyword, TDB_PARAM_TYPES
 from collections import defaultdict, namedtuple
@@ -177,26 +178,30 @@ def _tdb_grammar(): #pylint: disable=R0914
     Convenience function for getting the pyparsing grammar of a TDB file.
     """
     int_number = Word(nums).setParseAction(lambda t: [int(t[0])])
+    pos_neg_int_number = Word('+-'+nums).setParseAction(lambda t: [int(t[0])]) # '+3' or '-2' are examples
     # matching float w/ regex is ugly but is recommended by pyparsing
-    float_number = Regex(r'[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?') \
+    regex_after_decimal = r'([0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)'
+    float_number = Regex(r'[-+]?([0-9]+\.(?!([0-9]|[eE])))|{0}'.format(regex_after_decimal)) \
         .setParseAction(lambda t: [float(t[0])])
     # symbol name, e.g., phase name, function name
     symbol_name = Word(alphanums+'_:', min=1)
-    ref_phase_name = symbol_name = Word(alphanums+'_:()', min=1)
+    ref_phase_name = symbol_name = Word(alphanums+'_-:()/', min=1)
     # species name, e.g., CO2, AL, FE3+
-    species_name = Word(alphanums+'+-*', min=1) + Optional(Suppress('%'))
+    species_name = Word(alphanums+'+-*/_.', min=1) + Optional(Suppress('%'))
     # constituent arrays are colon-delimited
     # each subarray can be comma- or space-delimited
     constituent_array = Group(delimitedList(Group(OneOrMore(Optional(Suppress(',')) + species_name)), ':'))
     param_types = MatchFirst([TCCommand(param_type) for param_type in TDB_PARAM_TYPES])
     # Let sympy do heavy arithmetic / algebra parsing for us
     # a convenience function will handle the piecewise details
-    func_expr = Optional(float_number) + OneOrMore(SkipTo(';') \
+    func_expr = (float_number | ZeroOrMore(',').setParseAction(lambda t: 0.01)) + OneOrMore(SkipTo(';') \
         + Suppress(';') + ZeroOrMore(Suppress(',')) + Optional(float_number) + \
         Suppress(Word('YNyn', exact=1) | White()))
     # ELEMENT
     cmd_element = TCCommand('ELEMENT') + Word(alphas+'/-', min=1, max=2) + Optional(Suppress(ref_phase_name)) + \
         Optional(Suppress(OneOrMore(float_number))) + LineEnd()
+    # SPECIES
+    cmd_species = TCCommand('SPECIES') + species_name + Group(OneOrMore(Word(alphas, min=1, max=2) + Optional(float_number, default=1.0))) + Optional(Suppress('/') + pos_neg_int_number) + LineEnd()
     # TYPE_DEFINITION
     cmd_typedef = TCCommand('TYPE_DEFINITION') + \
         Suppress(White()) + CharsNotIn(' !', exact=1) + SkipTo(LineEnd())
@@ -209,8 +214,18 @@ def _tdb_grammar(): #pylint: disable=R0914
     cmd_defsysdef = TCCommand('DEFINE_SYSTEM_DEFAULT') + SkipTo(LineEnd())
     # DEFAULT_COMMAND
     cmd_defcmd = TCCommand('DEFAULT_COMMAND') + SkipTo(LineEnd())
+    # DATABASE_INFO
+    cmd_database_info = TCCommand('DATABASE_INFO') + SkipTo(LineEnd())
+    # VERSION_DATE
+    cmd_version_date = TCCommand('VERSION_DATE') + SkipTo(LineEnd())
+    # REFERENCE_FILE
+    cmd_reference_file = TCCommand('REFERENCE_FILE') + SkipTo(LineEnd())
+    # ADD_REFERENCES
+    cmd_add_ref = TCCommand('ADD_REFERENCES') + SkipTo(LineEnd())
     # LIST_OF_REFERENCES
     cmd_lor = TCCommand('LIST_OF_REFERENCES') + SkipTo(LineEnd())
+    # TEMPERATURE_LIMITS
+    cmd_templim = TCCommand('TEMPERATURE_LIMITS') + SkipTo(LineEnd())
     # PHASE
     cmd_phase = TCCommand('PHASE') + symbol_name + \
         Suppress(White()) + CharsNotIn(' !', min=1) + Suppress(White()) + \
@@ -228,12 +243,18 @@ def _tdb_grammar(): #pylint: disable=R0914
         Suppress(')') + func_expr.setParseAction(_make_piecewise_ast)
     # Now combine the grammar together
     all_commands = cmd_element | \
+                    cmd_species | \
                     cmd_typedef | \
                     cmd_function | \
                     cmd_ass_sys | \
                     cmd_defsysdef | \
                     cmd_defcmd | \
+                    cmd_database_info | \
+                    cmd_version_date | \
+                    cmd_reference_file | \
+                    cmd_add_ref | \
                     cmd_lor | \
+                    cmd_templim | \
                     cmd_phase | \
                     cmd_constituent | \
                     cmd_parameter
@@ -319,19 +340,31 @@ def _unimplemented(*args, **kwargs): #pylint: disable=W0613
     """
     pass
 
+def _process_species(db, sp_name, sp_comp, charge=0, *args):
+    """Add a species to the Database. If charge not specified, the Species will be neutral."""
+    # process the species composition list of [element1, ratio1, element2, ratio2, ..., elementN, ratioN]
+    constituents = {sp_comp[i]: sp_comp[i+1] for i in range(0, len(sp_comp), 2)}
+    db.species.add(Species(sp_name, constituents, charge=charge))
+
 def _setitem_raise_duplicates(dictionary, key, value):
     if key in dictionary:
         raise ValueError("TDB contains duplicate FUNCTION {}".format(key))
     dictionary[key] = value
 
 _TDB_PROCESSOR = {
-    'ELEMENT': lambda db, el: db.elements.add(el),
+    'ELEMENT': lambda db, el: (db.elements.add(el), _process_species(db, el, [el, 1], 0)),
+    'SPECIES': _process_species,
     'TYPE_DEFINITION': _process_typedef,
     'FUNCTION': lambda db, name, sym: _setitem_raise_duplicates(db.symbols, name, sym),
     'DEFINE_SYSTEM_DEFAULT': _unimplemented,
     'ASSESSED_SYSTEMS': _unimplemented,
     'DEFAULT_COMMAND': _unimplemented,
+    'DATABASE_INFO': _unimplemented,
+    'VERSION_DATE': _unimplemented,
+    'REFERENCE_FILE': _unimplemented,
+    'ADD_REFERENCES': _unimplemented,
     'LIST_OF_REFERENCES': _unimplemented,
+    'TEMPERATURE_LIMITS': _unimplemented,
     'PHASE': _process_phase,
     'CONSTITUENT': \
         lambda db, name, c: db.add_phase_constituents(
@@ -658,8 +691,19 @@ def write_tdb(dbf, fd, groupby='subsystem', if_incompatible='warn'):
         output += "ELEMENT {0} BLANK 0 0 0 !\n".format(element.upper())
     if len(dbf.elements) > 0:
         output += "\n"
-    for species in sorted(dbf.species):
-        output += "SPECIES {0} !\n".format(species.upper())
+    for species in sorted(dbf.species, key=lambda s: s.name):
+        if species.name not in dbf.elements:
+            # construct the charge part of the specie
+            if species.charge != 0:
+                if species.charge >0:
+                    charge_sign = '+'
+                else:
+                    charge_sign = ''
+                charge = '/{}{}'.format(charge_sign, species.charge)
+            else:
+                charge = ''
+            species_constituents = ''.join(['{}{}'.format(el, val) for el, val in sorted(species.constituents.items(), key=lambda t: t[0])])
+            output += "SPECIES {0} {1}{2} !\n".format(species.name.upper(), species_constituents, charge)
     if len(dbf.species) > 0:
         output += "\n"
     # Write FUNCTION block
@@ -770,8 +814,8 @@ def write_tdb(dbf, fd, groupby='subsystem', if_incompatible='warn'):
                                                         param_to_write.parameter_order,
                                                         exprx)
     if groupby == 'subsystem':
-        for num_elements in range(1, 5):
-            subsystems = list(itertools.combinations(sorted([i.upper() for i in dbf.elements]), num_elements))
+        for num_species in range(1, 5):
+            subsystems = list(itertools.combinations(sorted([i.name.upper() for i in dbf.species]), num_species))
             for subsystem in subsystems:
                 parameters = sorted(param_sorted[subsystem])
                 if len(parameters) > 0:
@@ -783,7 +827,7 @@ def write_tdb(dbf, fd, groupby='subsystem', if_incompatible='warn'):
                     for parameter in parameters:
                         output += write_parameter(parameter)
         # Don't generate combinatorics for multi-component subsystems or we'll run out of memory
-        if len(dbf.elements) > 4:
+        if len(dbf.species) > 4:
             subsystems = [k for k in param_sorted.keys() if len(k) > 4]
             for subsystem in subsystems:
                 parameters = sorted(param_sorted[subsystem])
